/**************************************************************/
/*                                                            */
/*   Program Name :  m5check                                  */
/*                                                            */
/*      Version:  1.74  2011-01-12                            */
/*                                                            */
/*  Copyright (c) 2003-2011 T.Kondo/NICT All Right Reserved   */
/*                                                            */
/**************************************************************/
/*

  Mark-5 data structure analysis

          T.Kondo 2003.03-11
                  2003.06-27 add non-parity 16bit-word mode
                  2003.10-13 add 64 track mode
                  2004.06-29 automatic both parity and non-parity modes check
                             becomes possible.
                             Results out was modified to display clearer
                  2004.09-20 head stack info was added
                  2004.09-25 default display mode was changed to monitor 
                               1st data block (8bytes) just after headder block
                  2004.10-20 debug in 1st data block pickup
                  2005.06-17 update due to the library restructuring
                  2006.09-26 add Video Converter # in output information
                  2008-01-08 debug for 2008 problem which is a problem 
                             of false detection of sync pattern of Mark5 
                             data time-labeled 2008            
                  2008-02-19 options '--help' and '--version'  were added 
                  2009-02-17 add no sync-block check mode. 
                             At coventional mode, OFST (the number of samples before
                             the first headder block) is added in monitor out items. 

                  2009-06-07 add Mark 5B check mode. 
                  2009-09-28 add initial bytes skip function. 
                  2009-09-30 minor update 
                  2010-10-21 minor update (add file size display in Mark5B analysis)
                  2011-01-12 printf("... %ld",fsize); => printf("... %lld", fsize); because
				             fsize is "int64_t" type.

    How to run
      m5check mk5file [mode] 
        or
      m5check mk5file options  -- for non sync block checking mode run
    
         where  mk5file -- Mark V data file name
                mode    -- data mode  (or display mode)
                             0: with parity and 8|16|32|64 bit-word mode 
                             1: without parity and 8|16|32|64 bit-word mode
                             2: to display using old display mode 
                             if mode is omitted, both modes are checked automatically

                 options 
				    -old -- force to default conventional (old) style run
                    -f  --- force to run with non synnc block check mode.
                             if one of following parameters is set, 
                             '-f' is omittable.
                             so '-f' can use when you want to run with non
                             sync block check mode with default parameters
                    -t[rack] ntrack --- # of tracks (8|16|32|64) 
                                        (default is 32)
                    -vlba --------- set VLBA mode  (default mark IV)
                    -p[arity]  --- set with parity mode (default non parity mode)
                    -nrzm  ------- set NRZM mode (default non NRZM)
                    -s samples(butes) --- set # of samples (or bytes) to be skipped initially


G:\IPVLBI\data\jiveexp3\d03l1_2.on        --- 64 track non-parity
G:\IPVLBI\data\jiveexp2\d03c1_2.mc        --- 32 track non-parity
G:\IPVLBI\data\tsev5\1-177-1300.dat       --- 16 track non-parity
G:\mark5\8-301-1814.dat
G:\IPVLBI\data\evlbi6\11-K178-1321  --- VLBA format with parity converted from K5
G:\mark5\m5test.dat
G:\IPVLBI\data\GSIcheck\k08006_wz_006-0827  --- Mark5 data taken in 2008
D:\IPVLBI\data\Mark5B\t09149a_Ho_149_060000.lba --- Mark5B data (8MHzX2bitX2ch) 
./m5data.m5b
D:\IPVLBI\data\testdata\test.dat -- K5 (just dummy)
*/

#include "libk5.h"      // common header for IP-VLBI utilities

#include "libmk5tok5.h"  // library for mark5 to k5 conversion (by T.Kondo)
#include "libvssp32.h"  // library for Mark5 to K5 conversion

#define PROGRAM_VERSION "Ver 1.74  2011-01-12"

typedef struct m5check_runp     // run parameters for m5check
{
    char mk5name[256];  
    int ptype;   // run mode 0: conventional mode  1: force to no-check mode
	             //          2: new mode (including Mark5B format analysis)
	             //             Now 2 is default
    int mode;
    int ntrack;
    int vlba;
    int mode_in_vlba;
    int parity;
    int nrzm;
    int dispmode;
    int skpbits;
} m5check_runp_t;

void param_get_m5check(int argc, char *argv[], m5check_runp_t *runp)                       
{
    int n;
    char cbuf[256];

    // initialize parameters

    //runp->ptype=0;  // run conventional mode
    runp->ptype=2;  // run with new mode (including Mark5B analysis)
    runp->mode=-1;
    runp->dispmode=1;
    runp->mk5name[0]='\0';
    runp->vlba=0;
    runp->mode_in_vlba=1;
    runp->parity=0;
    runp->nrzm=0;
    runp->ntrack=32;
    runp->skpbits=0;

    /* 1st parameter (mk5 file name) get */
    if (argc>=2){
       strcpy(runp->mk5name,*(argv+1));
    }

    /* 2nd param check whether it is started with "-" or not  */
    if (argc>=3){
        strcpy(cbuf,*(argv+2));
        if(cbuf[0] == '-') runp->ptype=1;
    }

    if(runp->ptype==0){ // conventional run mode
       /* data mode get */
       if (argc>=3){
          runp->mode= atoi(*(argv + 2 ));
          if(runp->mode==2){
              runp->dispmode=0;
              runp->mode=-1;
          }
       }
       goto exitend;
    }

    n=3;

    /* command parameters get */
    while(n <=argc) {
        strcpy(cbuf,*(argv+n-1));
        if(strncmp(cbuf,"-old",4)==0){  // force to conventional (old) run mode
            runp->ptype=0;
        }
        if(strncmp(cbuf,"-f",2)==0){  // force to set mode
            runp->ptype=1;
        }
        if(strncmp(cbuf,"-track",2)==0){  // # of track
            n++;
            if(n > argc) break;
            runp->ntrack = atoi(*(argv + n-1 ));
            if(runp->ptype!=1) runp->ptype=0;
       }
        if(strncmp(cbuf,"-skip",2)==0){  // # of samples to be skipped
            n++;
            if(n > argc) break;
            runp->skpbits = atoi(*(argv + n-1 ));
            if(runp->ptype!=1) runp->ptype=0;
        }
        if(strncmp(cbuf,"-parity",2)==0){  // parity
           runp->parity=1;
           if(runp->ptype!=1) runp->ptype=0;
        }
        if(strncmp(cbuf,"-nrzm",5)==0){  // NRZM
           runp->nrzm=1;
           if(runp->ptype!=1) runp->ptype=0;
        }
        if(strncmp(cbuf,"-vlba",5)==0){  // VLBA mode
           runp->vlba=1;
           if(runp->ptype!=1) runp->ptype=0;
        }
       n++;
    }

exitend:;

}

void print_usage(void)
{
    printf("m5check %s\n",PROGRAM_VERSION);
    printf("\n");
    printf("Usage: m5check m5file [mode]\n");
    printf("         or\n");
    printf("       m5check mk5file options  -- for non sync block checking mode run\n");
    printf("\n");
    printf("   where  m5file ---- Mark-5 data file name to cheak\n");
    printf("           mode ---- data mode\n");  
    printf("                     0: with parity 8|16|32|64 bit-word mode\n");
    printf("                     1: non parity  8|16|32|64 bit-word mode\n");
    printf("                     2: use old display style\n");
    printf("                     if mode is omitted, everything is checked automatically\n");
    printf("           options (any order)\n");
    printf("             -old -- force to default conventional (old) style run\n");
    printf("             -f  --- force to run with non synnc block check mode\n");
    printf("                     if one of following parameters is set, \n");
    printf("                     '-f' is omittable.\n");
    printf("                     so '-f' can use when you want to run with non\n");
    printf("                     sync block check mode with default parameters\n");
    printf("             -t[rack] ntrack --- # of tracks (8|16|32|64) \n");
    printf("                                 (default is 32)\n");
    printf("             -vlba -------- set VLBA mode  (default mark IV)\n");
    printf("             -p[arity]  --- set with parity mode\n");
    printf("                            (default non parity mode)\n");
    printf("             -nrzm  ------- set NRZM mode (default non NRZM)\n");
    printf("             -s[kip] samples --- set # of samples to be skipped initially\n");
    printf("\n");
}


int main(int argc, char *argv[])
{
       int i,iret;
       //int header[160];  // header  5 X 32ch
       int header[320];  // header  5 X 64ch    2003-10-13
       int istart[6];
       int validtrk[64], trktable[64], hstack_tbl[64]; 
       int vcntabl[64];
       int ntrack;
       int mode,imode;
       int parity, nrzm, bitsframe;
       int vlba;
       int mode_in_vlba;
       int skpbits, skpbytes;
       int dispmode;   // display mode 0: old style  1: new style
                                                      // 1 st data block display
       double fsampl;
	   double fbps;      // for Mark5B
	   int iuid,tvgflag; // for Mark5B
       int framsec;
       double tframe,dlength;
       int64_t fsize;
       char mk5name[256];
       double telapsed;           // to measure processing time
       time_t tstart, tfinish;    //           "
       m5check_runp_t runp;

       if(argc == 1 || (argc == 2 && strncmp(*(argv+1),"--help",6)==0 )){ // 2008-02-19
          print_usage();
          exit(0);
       }
       if(argc == 2 && strncmp(*(argv+1),"--ver",5)==0 ){ // 2008-02-19
          printf("\n");
          printf("m5check %s\n",PROGRAM_VERSION);
          printf("\n");
          exit(0);
       } 

       param_get_m5check(argc, argv, &runp);                       
       /* parameters set up */
       mode=runp.mode;
       dispmode=runp.dispmode;
       strcpy(mk5name,runp.mk5name);

       /* Program Name Display */
       printf("***********************************************************\n");
       printf("*    Mark-5 data structure analysis                       *\n");
       printf("*      %s  by  T.KONDO/NICT             *\n",PROGRAM_VERSION);
       printf("*                                                         *\n");
       if(runp.ptype!=2){
       if(runp.ptype==0){
		   if(mode==0){
		   printf("*      NRZM, with parity, 8|16|32|64 bit-word mode        *\n");
		   } else if(mode==1){
		   printf("*      NRZL, non parity, 8|16|32|64 bit-word mode         *\n");
		   } else {
		   printf("*      automatic check both parity and non-parity modes   *\n");
		   }
		   } else {
		   printf("*      No synck-block check mode with parameters below    *\n");
		   printf("*         # of tracks : %2d                                *\n",runp.ntrack);
		   if(runp.vlba==0){
		   printf("*         Format : Mark-IV                                *\n");
		   } else {
		   printf("*         Format : VLBA                                   *\n");
		   if(runp.mode_in_vlba==1){
		   printf("*         Frame start : middle of headder block           *\n");
		   } else {
		   printf("*         Frame start : begining of headder block         *\n");
		   }
       }
       if(runp.parity==0){
       printf("*         Parity : non parity                             *\n");
       } else {
       printf("*         Parity : with parity                            *\n");
       }
       if(runp.nrzm==1){
       printf("*         Data coding : NRZM                              *\n");
       } else {
       printf("*         Data coding : no coding                         *\n");
       }
       printf("*         # of samples (or bytes) to be skipped : %6d  *\n",runp.skpbits);
       }
	   } else { // runp.ptype=2

	   }
       printf("***********************************************************\n");
       printf("\n");

       tstart= time(NULL);

       /* First check Mark 5B format (added on 2009-06-06 */

	   printf("1st STEP: Checking Mark5B format\n");

       //iret=mk5b_dat_anal(mk5name, 0, 10, istart);
       //iret=mk5b_dat_anal(mk5name, 1, 10, istart);
       iret=mk5b_dat_anal(mk5name, 2, 10, istart,&iuid,&tvgflag,&fbps,&framsec,&fsize,&tframe,&dlength);

	   if(iret == 1){  // this is Mark5B format
		   printf("\n");
		   printf("**************** SUMMARY of DATA FORMAT ANALYSIS****************\n");
		   printf(" File Name   : %s\n",mk5name);
		   printf(" Data Format : Mark-5B\n");
           //printf(" File Size   : %ld bytes\n",fsize);       
           printf(" File Size   : %lld bytes",fsize);       // 2011-01-12
           printf("    Total # of Frames: %f\n",tframe);  // int64 type variable and other typ in 1 line
                                                           // results in 0 value for 2nd one in WIN32
                                                           // So, seprated in 2 lines
		   printf(" 1st Header Information\n");
		   printf("   Time (MJD HH:MM:SS.SSSS) : %03d %02d:%02d:%02d.%04d\n",
			   istart[1],istart[2],istart[3],istart[4],istart[5]);
		   printf("   User Specified (16 bits) : %04X\n",(unsigned short int)iuid);
		   printf("   TVG data flag            : %d\n",tvgflag);
		   printf(" Data Rate (Mbps) : %6.1f\n",fbps/1.0e6);
           printf(" Frames/sec       : %d\n",framsec);       
           printf(" Data Length (sec): %f\n",dlength);       
           printf("****************************************************************\n");
		   exit(0);
	   }

	   printf("  ==> Not Mark5B format\n");


	   /* check Mark 5 conventional format */
	   printf("\n");
	   printf("2nd STEP: Checking Mark5 (old style) format\n");


       if(runp.ptype==1) {// force to no sync check mode
	      printf("   ==>  force to check with no-synch detection mode\n");
          vlba=runp.vlba;
          parity=runp.parity;
          nrzm=runp.nrzm;
          ntrack=runp.ntrack;
          mode_in_vlba=runp.mode_in_vlba;
          if(vlba==1) imode=0;
          skpbits=runp.skpbits;

          iret=mk5_dat_anal_without_sync(mk5name, validtrk, trktable, hstack_tbl,
                  header, &fsampl, istart, ntrack, &bitsframe, parity, nrzm, dispmode, 
                  vlba, mode_in_vlba, skpbits, vcntabl); // 2009-02-16
          goto skiptohere;
       }

       /* Mark5 data structure check */

       skpbytes=runp.skpbits;  //2009-09-28

       if(mode==0 || mode ==1){
          if(mode==0){
             parity=1; nrzm=1;
             imode=0;
          } else {
             parity=0; nrzm=0;
             imode=1;
          }
          iret=mk5_dat_anal(mk5name, validtrk, trktable, hstack_tbl,
              header, &fsampl, istart, &ntrack, &bitsframe, parity, nrzm, dispmode,
			  skpbytes, //2009-09-28
              vcntabl);
       } else {
           parity=0; nrzm=0;
           for(i=0; i<2; i++){
              if(i==0){
                 imode=1;
                 printf("Non-parity mode is assumed first in data format analysis..\n");
              } 
              if(i==1){
                 imode=0;
                 parity=1; nrzm=1;
                 printf("No meaning data are found!\n");
                 printf("Parity mode is assumed in data format analysis..\n");
              }  
              if(mk5_dat_anal(mk5name, validtrk, trktable, hstack_tbl, header, &fsampl, 
                 istart, &ntrack, &bitsframe, parity, nrzm, dispmode,
				 skpbytes, //2009-09-28
                 vcntabl) >= 0) break; //  meaning data found
           }  
       } 
       if(ntrack ==0){
           printf("No meaning data are found!\n");
           return 0;
       }

skiptohere:

       printf("\n");
       printf("**************** SUMMARY of DATA FORMAT ANALYSIS****************\n");
       printf(" File Name   : %s\n",mk5name);
	   printf(" Data Format : Mark-5\n");
       printf(" # of tracks : %d\n",ntrack);
       //printf(" # of bits (tracks) per word is %d\n",ntrack);
       //if(imode==0){
       //    printf(" DATA mode   : NRZM  with parity\n");
       //} else {
       //    printf(" DATA mode   : NRZL  without parity\n");
       //}
       if(nrzm==1){
           printf(" DATA mode   : NRZM");
		   if(parity==1){
              printf("  with parity\n");
		   } else {
              printf("  without parity\n");
		   }
       } else {
           printf(" DATA mode   : NRZL");
		   if(parity==1){
              printf("  with parity\n");
		   } else {
              printf("  without parity\n");
		   }
       }
       printf(" bits/frame  : %d ",bitsframe);
       if(bitsframe==20160) printf(" (This is VLBA Format)");
       if(bitsframe==20000) printf(" (This is Mark-IV Format)");
       printf("\n");
       if(istart[0]==-1){ // VLBA format
          printf(" 1st header time (MJD HH:MM:SS.SSSS) : %03d %02d:%02d:%02d.%04d\n",
           istart[1],istart[2],istart[3],istart[4],istart[5]);
       } else { // Mark-IV format
          printf(" 1st header time (Y/DDD HH:MM:SS.SSS) : %d/%03d %02d:%02d:%02d.%03d\n",
           istart[0],istart[1],istart[2],istart[3],istart[4],istart[5]);
       }
       printf(" Bit position and track# table : \n");
       printf("\n      bit pos  00 01 02 03 04 05 06 07 08 09 10 11 12 13 14 15\n");
       printf("      Validity");
       for(i=0; i<16; i++) printf(" %2d",validtrk[i]);
       printf("\n");
       printf("      Track#  ");
       for(i=0; i<16; i++) printf(" %2d",trktable[i]);
       printf("\n");
       printf("      HDstack#");
       for(i=0; i<16; i++) printf(" %2d",hstack_tbl[i]);
       printf("\n");
       printf("      VideCon#");
       for(i=0; i<16; i++) printf(" %2d",vcntabl[i]);
       printf("\n");
       if(ntrack > 16){
       printf("\n      bit pos  16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31\n");
       printf("      Validity");
       for(i=16; i<32; i++) printf(" %2d",validtrk[i]);
       printf("\n");
       printf("      Track#  ");
       for(i=16; i<32; i++) printf(" %2d",trktable[i]);
       printf("\n");
       printf("      HDstack#");
       for(i=16; i<32; i++) printf(" %2d",hstack_tbl[i]);
       printf("\n");
       printf("      VideCon#");
       for(i=16; i<32; i++) printf(" %2d",vcntabl[i]);
       printf("\n");
       }
       if(ntrack > 32){
       printf("\n      bit pos  32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47\n");
       printf("      Validity");
       for(i=32; i<48; i++) printf(" %2d",validtrk[i]);
       printf("\n");
       printf("      Track#  ");
       for(i=32; i<48; i++) printf(" %2d",trktable[i]);
       printf("\n");
       printf("      HDstack#");
       for(i=32; i<48; i++) printf(" %2d",hstack_tbl[i]);
       printf("\n");
       printf("      VideCon#");
       for(i=32; i<48; i++) printf(" %2d",vcntabl[i]);
       printf("\n");
       printf("\n      bit pos  48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63\n");
       printf("      Validity");
       for(i=48; i<64; i++) printf(" %2d",validtrk[i]);
       printf("\n");
       printf("      Track#  ");
       for(i=48; i<64; i++) printf(" %2d",trktable[i]);
       printf("\n");
       printf("      HDstack#");
       for(i=48; i<64; i++) printf(" %2d",hstack_tbl[i]);
       printf("\n");
       printf("      VideCon#");
       for(i=48; i<64; i++) printf(" %2d",vcntabl[i]);
       printf("\n");
       }
       printf("\n");
       printf("****************************************************************\n");

 /*
       for(i=0; i<32; i++){
           printf("%02d ",i);
           for (k=0; k<5; k++){            
                 printf("%08x ",rvsint(header[5*i+k]));
           }
           printf("\n");
       }
*/
       tfinish= time(NULL);
       telapsed = difftime(tfinish,tstart);
    //   printf("Time elapsed (sec) for One obs process is %f\n",telapsed);


       return 0;
}
